
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
import matplotlib
import subprocess 
import mesaPlot as mp
import os
import sys
import glob
import pickle
import multiprocessing as mup
import itertools
import scipy
from scipy.spatial import distance
import matplotlib.mlab as mlab
from matplotlib.pyplot import cm
import numbers
import hashlib
import contextlib
from itertools import cycle

from matplotlib import rc
import matplotlib.patches as patches
rc('text', usetex=True)

try:
    plt.style.use('./mesa.mplstyle')
except OSError:
    pass #On cluster skip file
fig_output='./'

NONE=0
PISN_IND=1
PPISN_IND=2
CC_IND=3
UPPER_IND=4

mpplot = mp.plot()

Yellow = "#DDDD77"
Green = "#88CCAA"
Blue = "#77AADD"
Red = "#FFA9A9"

Maroon='#590A30'
Green2='#90AA3C'
Orange='#EF6125'

sp=200
dbg=False

mheintl = r'$\rm{Initial\, mass\,of\, helium\, \left(M_{He}\,[M_{\odot}]\right)}$'
mcintl = r'$\rm{CO\,core\, mass\, \left(M_{CO}\,[M_{\odot}]\right)}$'
mbhl = r'$\rm{Black\, hole\, mass\, \left(M_{BH}\,[M_{\odot}]\right)}$'

def md5(fname):
    hash_md5 = hashlib.md5()
    with open(fname, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()

def fixbasename(base):
    if not '/' in base:
        return base
    if base.endswith('/'):
        return os.path.basename(os.path.dirname(base))
    return os.path.basename(base)
    
def overplot_ligo(fig,ax,ylim):
    x=np.genfromtxt("./ligo_events.csv",names=True,dtype=None)
    #ax2=fig.add_subplot(122)
    x1=np.arange(len(x['m1'])*1.0)/len(x['m1'])
    x2=x1*10.0
    x3=0.1+x1*10.0
    ind=x['m1']>0.0
    ax.set_yticklabels([])
    ax.set_xticks(x2[ind])
    
    labels=[i.decode().upper() for i in x['name'][ind]]
    
    ax.set_xticklabels(labels, rotation=90,fontsize=18)
    ax.tick_params(axis='x', which='minor', bottom=False,top=False)
    ax.set_ylim(ylim)
    ax.errorbar(x2[ind],x['m1'][ind],yerr=[x['m1_minus'][ind],x['m1_plus'][ind]], fmt='o',markersize=int(np.sqrt(200.0)),ecolor='k')
    ax.errorbar(x3[ind],x['m2'][ind],yerr=[x['m2_minus'][ind],x['m2_plus'][ind]], fmt='o',markersize=int(np.sqrt(200.0)),ecolor='k')
    return
    
    
def getCloseToBurnOnsetModNum(m,offset=2.9):
    # find location where the nuclear luminosity is 3.5dex what it was during the He core burning phase.
    # returns the first model number ar which this condition is met.
    # by trial and error, this returns approximately the location before a "pulse"
    i = np.argmin(np.absolute(m.hist.star_age -1e5))
    logLmin = m.hist.log_Lnuc[i]
    logLnuc = m.hist.log_Lnuc[i::]
    c12_c12 = m.hist.c12_c12[i::]
    mod_num = m.hist.model_number[i::]
    j = c12_c12 > logLmin+offset
    
    if np.count_nonzero(j) > 0:
        n = np.min(mod_num[j])
    else:
        n = mod_num[np.argmax(logLmin)]
    
    return n

def findIgnitionPoint(m):
    min_num = int(getCloseToBurnOnsetModNum(m))
    max_num = int(np.minimum(min_num+2500,m.hist.model_number[-1]))
    # load model numbers for 2500 timesteps starting from the timestep
    # returned by getCloseToBurnOnsetModNum
    # these will be used to browse the other data
    k0 = -1
    n0 = -1
    for n in range(min_num,max_num):
        #loop over timesteps (x-direction in a Kippenhahn)
        # print n
        for k in range(1,21):
            # loop over the vertical direction in a Kippenhahn)
            # print k
            ind = m.hist.model_number==n
            if m.hist.data['burn_type_'+str(k)][ind]>=9 and m.hist.data['burn_qtop_'+str(k)][ind]<=0.5:
                k0 = k
                n0 = n
                break
    return n0,k0

def findIgnition(m,ignPoint, ignQtop):
    if ignPoint > 0:
        ind = m.hist.model_number==ignPoint
        qtop = m.hist.data['burn_qtop_'+str(ignQtop)][ind]
        ignq = qtop[0]
        igm = qtop[0]*m.hist.star_mass[ind][0]
    else:
        ignq = -1
        igm = -1

    return ignq,igm

def history(folder):
    m=mp.MESA()
    m.log_fold=os.path.join(folder,'LOGS')
    comass= np.nan
    hemass= np.nan
    ignPoint= -1
    ignq = -1
    ignm = -1
    intm = 0.0
    intz = -1
    c12cent_he = np.nan
    comassfail=False
    res = {}
    res2 = {}
    c12cent_he = np.nan
    try:
        m.loadHistory()
        m.hist.data = m.hist.data[m.hist.model_number<7000]
    except OSError:
        pass
    else:
        comass = np.nanmax(m.hist.c_core_mass)
        ind = np.nanargmax(m.hist.c_core_mass)
        if comass==0.0:
            ind=m.hist.center_c12>0.1
            comass=m.hist.total_mass_c12[ind][-1]+m.hist.total_mass_o16[ind][-1]
            ind=(m.hist.model_number==m.hist.model_number[ind][-1])
            comassfail=True
        
        hemass = m.hist.star_mass[ind]
        c12cent_he = np.nanmax(m.hist.center_c12)
        ignPoint, ignQtop = findIgnitionPoint(m)
        ignq, ignm = findIgnition(m,ignPoint, ignQtop) 
        # Find the point after core helium burning has finished
        indpche=(m.hist.mix_type_1==0)&(m.hist.mix_qtop_1>0.5)

        for k in m.hist.data.dtype.names:
            res[k+'_ign'] = m.hist.data[k][ignPoint]
            res2[k+'_posthe'] = m.hist.data[k][indpche][0]

        for k in m.hist.head.dtype.names:
            res[k+'_ign'] = m.hist.head[k]            

        # not in new version of mesa
        intm = m.hist.star_mass[0]
        #intz = m.hist.initial_z
 
    return {'comass_pre':comass,'hemass_pre':hemass,'intm':intm,'ignPoint':ignPoint, 'ignq':ignq,'ignm':ignm,**res,'comassfail':comassfail,'c12cent_he':c12cent_he,**res2}


def finalProf(folder):
    G=6.67428*10**-8
    msun=1.9892*10**33
    rsun=6.9598*10**10
    m=mp.MESA()
    bhmass = np.nan
    failed = False
    failedBE = False
    m.log_fold=os.path.join(folder,'LOGS')
    
    m.loadHistory()
    maxR = np.max(m.hist.log_R_vesc)
    
    m.hist.data = m.hist.data[-1]
    
    res = {}
    for k in m.hist.data.dtype.names:
        res[k+'_end'] = m.hist.data[k]

    for k in m.hist.head.dtype.names:
        res[k+'_end'] = m.hist.head[k]
    
    fin = m.hist.M_below_vesc
    bhmass = m.hist.M_below_vesc
    
    
    #res2={}
    try:
        m.loadProfile(f=os.path.join(folder,'final_prof.prof'))
    except OSError:
        failed=True
    #else:
    #    for k in m.prof.data.dtype.names:
    #        res[k+'_prof_end'] = m.prof.data[k]

     #   for k in m.prof.head.dtype.names:
     #       res[k+'_prof_end'] = m.prof.head[k]
        
    if not failed:
        dm=m.prof.dm
        m_grav=m.prof.mass*msun
        r=10**(m.prof.logR)*rsun
        be=G*dm*m_grav/r
        try:
            bhmass = m.prof.mass[be>10**48][0]
        except IndexError:
            try:
                bhmass = m.hist.M_below_vesc
            except AttributeError:
                bhmass = np.nan
                failedBE = True
                failed=True
        if not np.isfinite(bhmass):
            bhmass = m.hist.M_below_vesc
            failedBE = True
        
    return {'bhmass':bhmass,'final_mass':fin,'failed':failed,'failedBE':failedBE,'max_r_bound':maxR,**res}


def preSN(folder):
    m=mp.MESA()
    emass = np.nan
    try:
        m.loadProfile(f=os.path.join(folder,'LOGS','preSN.data'))
    except OSError:
        return {'emass':emass} 
    
    p=m.prof.pgas+m.prof.prad
    t=10**m.prof.logT 

    lnt = m.prof.logT/np.log10(np.e)
    lnrho = m.prof.logRho/np.log10(np.e)

    dpdlnt = np.gradient(p, lnt)
    dpdlnd = np.gradient(p, lnrho)
    cv = np.gradient(m.prof.energy,lnt)/t

    chit = dpdlnt/p
    chirho = dpdlnd/p

    y = 10**m.prof.logRho * t * cv
    A = p * chit/y

    g1 = chit * A + chirho
    
    ind = (g1<4.0/3.0)&(m.prof.mass<m.prof.c_core_mass)
    if np.count_nonzero(ind) > 1:
        mm = m.prof.mass[(g1<4.0/3.0)&(m.prof.mass<m.prof.c_core_mass)]
        emass = mm[0]-mm[-1]
    else:
        emass = np.nan

    res = {}

    eind = np.argmax(m.prof.energy)

    for k in m.prof.data.dtype.names:
        res[k+'_presn'] = m.prof.data[k][eind]

    for k in m.prof.head.dtype.names:
        res[k+'_presn'] = m.prof.head[k]

    return res


def hydro1(folder):
    m=mp.MESA()
    emass = np.nan
    try:
        m.loadProfile(f=os.path.join(folder,'LOGS','preSN.data'))
    except OSError:
        return {} 
    
    eind = np.argmax(m.prof.energy)
    mass = m.prof.mass[eind]

    try:
        m.loadProfile(f=os.path.join(folder,'LOGS','prehydro_prof001.data'))
    except OSError:
        return {}    

    ind = np.argmin(np.abs(m.prof.mass-mass))
    res = {}

    for k in m.prof.data.dtype.names:
        res[k+'_prehydro'] = m.prof.data[k][ind]

    for k in m.prof.head.dtype.names:
        res[k+'_prehydro'] = m.prof.head[k]

    return res


def inlist(folder):
    inlist_name = os.path.join(folder,'inlist_extra')

    if not os.path.isfile(inlist_name):
        return {}

    def extractValue(l):
        if '!' in l:
            l = l[:l.index('!')]
        return l.split('=')[-1].strip().replace('d','e')

    res = {}
    with open(inlist_name,'r') as f:
        for line in f.readlines():
            if 'initial_mass' in line: 
                res['initial_mass'] = extractValue(line)

    rate_vars = os.path.join(folder,'rate_varitation_factors.txt')
    if os.path.exists(rate_vars):
        res['rate_vars'] = np.genfromtxt(rate_vars,names=['id','sigma'])
    else:
        res['rate_vars'] = None

    return res



def readOutput(base,mass,reset=True):
    
    if int(mass) == mass:
        folder = os.path.join(str(base),str(int(mass)))   
    else:
        folder = os.path.join(str(base),str(mass))  
    
    f=os.path.join(folder,'run.log')        
    
#    if not os.path.exists(folder) and int(mass) == mass:
#        folder = os.path.join(str(base),str(int(mass)))  
#        f=os.path.join(folder,'run.log')        
    
    
    if dbg: print(folder)
    runcache = md5(f)
        
    z={}
    if not reset:
        try:
            with open(os.path.join(folder,'dump.txt'),'rb') as ff:
                z = pickle.load(ff)
                z['base'] = fixbasename(base)
        except (OSError, pickle.UnpicklingError):
            if dbg: print("Failed loading "+str(folder))   
        except FileNotFoundError:
            pass
    
        if 'runcache' in z:
            print(base,mass,z['runcache'],runcache)
            if z['runcache'] == runcache:
                print("cached ",base,mass)
                return z
    
    print("not cached ",base,mass,runcache)
    fe=subprocess.call(['grep','-qF','termination code: fe_core_infall_limit',f])
    pisn=subprocess.call(['grep','-qF','Entire star is expanding above the escape velocity, PISN!',f])
    job_killed=subprocess.call(['grep','-qF','job killed',f])
    pulsed=subprocess.call(['grep','-qF','relax_composition: num_steps_to_use',f])
    max_model=subprocess.call(['grep','-qF','termination code: max_model_number',f])
    min_timestep=subprocess.call(['grep','-qF','termination code: min_timetsep_limit',f])

    symbol=UPPER_IND
    if max_model == 0:
        symbol=UPPER_IND
    elif pulsed == 0:
        if fe == 0:
            symbol=PPISN_IND 
        else:
            symbol=UPPER_IND
    elif pisn == 0:
        symbol = PISN_IND 
    else:
        if fe == 0 :
            symbol = CC_IND
        else:
            symbol = UPPER_IND 

    if dbg: print(base,mass,symbol,fe,pisn,job_killed,pulsed,max_model,min_timestep)
        
    d1={}
    d2={}
    d3={}
    d4={}
    d5={}

    d1 = preSN(folder)
    d2 = finalProf(folder)
    d3 = history(folder) 
    d4 = inlist(folder)
    d5 = hydro1(folder)

    data = {**d1,**d2,**d3,**d4,**d5}

    data['base'] = fixbasename(base)
    data['intmass'] = mass
    data['symbol'] = symbol
    data['last_update'] = -1
    data['runcache'] = runcache
    print("Saving ",base,mass,data['runcache'])
    
    try:
        with open(os.path.join(folder,'dump.txt'),'wb') as ff:
            pickle.dump(data,ff)
    except OSError:
        if dbg: print("Failed saving "+str(folder))

    return data


def procAll(bases,masses):
    bases = list(set(bases))
    with mup.Pool(processes=4) as pool:
        _ = pool.starmap(readOutput,itertools.product(bases,masses))



def readAll(bases,masses,resetAll=False,reset=False,savefile='alldata.npy'):
    data = {}
    dt = {}
    
    bases = list(set(bases))
    bases.sort()

    if resetAll or not os.path.isfile(savefile):
        for i in bases:
            data[i] = {}
            for j in masses:
                data[i][j] = {}
                try:
                    data[i][j] = readOutput(i,j,reset)
                except Exception as e:
                    print("Error in ",str(i),str(j),repr(e))
                    continue
                for k,v in data[i][j].items():
                    if 'mix_type' in k or 'qtop' in k or 'burn_type' in k or 'relr' in k:
                        continue
                    
                    if isinstance(v,str):
                        t = 'U25'
                    else:
                        t = float
                    dt[k] = t

                if dbg: print(i,j)
        #Now repack data into an array

        dtype = np.dtype([(k,v) for k,v in dt.items()])

        l1 = len(masses)
        size = l1 * len(bases)
        arr = np.zeros(size,dtype=dtype)
        for idx,i in enumerate(bases): 
            for jdx,j in enumerate(masses):
                index = (idx*l1) + jdx
                for k in dtype.names:
                    if k not in data[i][j]:
                        pass
                    else:
                        try:
                            arr[index][k] = data[i][j][k]
                        except ValueError:
                            if dbg: print("Failed to convert "+str(k)+' "'+str(data[i][j][k])+'"')
                            arr[index][k] = 0
        np.save(savefile, arr)
    else:
        arr = np.load(savefile,allow_pickle=True)

    return arr


def plotBox(ax,xmin,xmax,ymin,ymax,color,alpha=0.25):
    ax.add_patch(    patches.Rectangle(
        (xmin,ymin),   # (x,y)
        xmax-xmin,          # width
        ymax-ymin,          # height
        facecolor=color,
        alpha=alpha
    ))

def plotLine(data,bases,xax,yax,labels,xrng=None,yrng=None,xlabel=None,ylabel=None,name='',loc=0,
                upper=False,colors=False,
                maxbh=False,single=False,info=True,ligo=True,
                xlog=False,ylog=False,ishe=False,skip=[],pisn=True,ppisn=True,cc=True,lines=True,
                cmap=cm.plasma,colorbar=False,legend=True,smallco=True,text=False,fs=20,paper=True,show=True,
            labely=50,figsize=(12,9),legendcol=1):
    

    if ligo:
        fig, (ax, ax1) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[3, 1],'wspace':0.025, 'hspace':0.05},figsize=figsize)
    else:
        fig, ax = plt.subplots(1,1,figsize=figsize)

    if isinstance(xax,str):
        ishe = ('hemass' in xax) or ('initial_mass' in xax)
    
    fig.set_tight_layout(False)
    maxlist=[]
    
    color=iter(cmap(np.linspace(0,1.0,len(bases)))) # Try to avoid the final yellow color
    
    linestyle_tuple = [
        'solid','dashed','dotted','dashdot']
    licycle = cycle(linestyle_tuple)    
        
    for jdx,j in enumerate(bases):
        print(j)
        if j in skip:
            continue

        ind = (data['base'] == j)

        if isinstance(xax,str): 
            xaxis = data[xax][ind]
        else:
            xaxis = xax[ind]

        if isinstance(yax,str): 
            yaxis = data[yax][ind]
        else:
            yaxis = yax[ind]

        if yaxis.dtype == 'U25':
            yaxis[np.where(yaxis=='')] = np.nan
            yaxis = yaxis.astype(np.float)
        if xaxis.dtype == 'U25':
            xaxis[np.where(xaxis=='')] = np.nan  
            xaxis = xaxis.astype(np.float)

        if xlog:
            xaxis = np.log10(xaxis)

        if ylog:
            yaxis = np.log10(yaxis)

        symbol = data['symbol'][ind]

        yaxis[symbol==PISN_IND] = 0.0

        indco=np.zeros(np.size(xaxis),dtype='bool')
        indco[:]=True
        indfailed = (data['failed'][ind]==1)&np.logical_not(symbol==PISN_IND)
        
        if not smallco:
            indco=xaxis>10.0
        
        indco = indco&np.logical_not(indfailed)
        
        c=matplotlib.colors.rgb2hex(next(color))
        if lines:
            if upper:
                l,=ax.plot(xaxis[indco],yaxis[indco],label=labels[jdx],c=c,ls=next(licycle))
            else:
                ind=(symbol!=UPPER_IND)
                l,=ax.plot(xaxis[ind&indco],yaxis[ind&indco],label=labels[jdx],c=c,ls=next(licycle))

            line_col = c
        else:
            line_col=c
            
            
        #pisn
        if pisn:
            ind1=(symbol==PISN_IND)&(indco==True)
            ax.scatter(xaxis[ind1],[0.0]*np.count_nonzero(ind1),marker='x',c=line_col,s=sp)
        #PPISN
        if ppisn:
            ind2=(symbol==PPISN_IND)&(indco==True)
            ax.scatter(xaxis[ind2],yaxis[ind2],marker='o',c=line_col,s=sp)
        #Fe CC 
        ind3=(symbol==CC_IND)&(indco==True) #need ind3 even if we dont plot CC
        if cc:
            ax.scatter(xaxis[ind3],yaxis[ind3],marker='+',c=line_col,s=sp)
        #Upper limit
        if upper:
            ind=(symbol==UPPER_IND)&(indco==True)
            ax.scatter(xaxis[ind],yaxis[ind],marker='v',c=line_col,s=sp)

        if info:
            ind=(symbol==UPPER_IND)&(indco==True)
            print("fail",j+'/{'+','.join([str(i) for i in data['i'][ind]])+'}')

            try:
                m1=xaxis[ind2][0]
            except IndexError:
                m1 = 0

            try:
                m2=xaxis[ind1][0]
            except IndexError:
                m2 = 0

            try:
                print("First ",ishe,m1,m2)
            except IndexError:
                print("First failed")

        if jdx==0 and colors:
            plotBox(ax,60.349333706078134, 150.0,-5.0,100.0,Yellow)
            plotBox(ax,10.0 ,37.11426254752192,-5.0,100.0,Blue)
            plotBox(ax,37.11426254752192, 60.349333706078134,-5.0,100.0,Green)
            
            
            ax.text(26,labely-2,'Core \nCollapse',fontsize=fs-7)
            ax.text(43.5,labely,'Pulsations',fontsize=fs-7)
            ax.text(62,labely-2,'Pair \nInstability',fontsize=fs-7)
            
        
        if np.count_nonzero(yaxis[ind2|ind3]) > 0:
            if not j=='5dm3':
                maxlist.append(np.nanmax(yaxis[ind2|ind3]))
             
        if single:
            ax.plot([0,1],[0,1],c='k',label='Farmer+ In prep')
            break
    
    if xrng is not None:
        ax.set_xlim(xrng)

    if yrng is not None:
        ax.set_ylim(yrng)
    
    ax.tick_params(axis='x',which='minor')
    ax.tick_params(axis='y',which='minor')


    if xlabel is None:
        ax.set_xlabel(r'$\rm{M}\,[M_\odot]$',fontsize=fs)
    else:
        ax.set_xlabel(xlabel,fontsize=fs)

    if ylabel is None:
        ax.set_ylabel(mbhl,fontsize=fs)
    else:
        ax.set_ylabel(ylabel,fontsize=fs)

    ax.xaxis.set_tick_params(labelsize=fs-2)
    ax.yaxis.set_tick_params(labelsize=fs-2)
    
    if ligo:
        overplot_ligo(fig,ax1,ax.get_ylim())

    if maxbh:
        print(maxlist)
        #plotBox(ax,*ax.get_xlim(),np.min(maxlist),np.max(maxlist),Red)
        ax.plot([*ax.get_xlim()],[np.min(maxlist),np.min(maxlist)],'--',c='black',linewidth=3)
        ax.plot([*ax.get_xlim()],[np.max(maxlist),np.max(maxlist)],'--',c='black',linewidth=3)
    
    if legend:
        ax.legend(loc=loc,fontsize=fs-10,ncol=legendcol)

    if len(name):
        if paper:
            name = fig_output+name
        plt.savefig(name,dpi=300)
        
    if show:
        plt.show()
    

def plot2d(data,bases,xaxis,yaxis,zaxis,xlabel=None,ylabel=None,zlabel=None,xlog=False,ylog=False,zlog=False,
           contours=False,name='',xrng=None,yrng=None,labels=False,fs=30,hatch=0,hs='\\',figsize=(12,9),lpad=5,cm='plasma',alpha=1.0):
    
    fig=plt.figure(figsize=figsize)
    ax=fig.add_subplot(111)
    ax.set_yscale("log", nonposy='clip')

    if isinstance(yaxis,str):
        yax = data[yaxis]
    else:
        yax = yaxis

    if isinstance(xaxis,str):
        xax = data[xaxis]
    else:
        xax = xaxis

    if isinstance(zaxis,str):
        zax = data[zaxis]
    else:
        zax = zaxis

    if yax.dtype == 'U25':
        yax[np.where(yax=='')] = np.nan
    if xax.dtype == 'U25':
        xax[np.where(xax=='')] = np.nan
    if zax.dtype == 'U25':
        zax[np.where(zax=='')] = np.nan    

    yax = yax.astype(np.float)
    xax = xax.astype(np.float)
    zax = zax.astype(np.float)

    if ylog:
        yax=np.log10(yax)

    if xlog:
        xax=np.log10(xax)

    if zlog:
        zax=np.log10(zax)

    symbol = (data['symbol']!=UPPER_IND) & (data['symbol']!=PISN_IND)

    indBases = np.isin(data['base'],bases)

    if xrng is None:
        xrng = np.nanmin(xax[indBases]),np.nanmax(xax[indBases])

    if yrng is None:
        yrng= np.nanmin(yax[indBases]),np.nanmax(yax[indBases])

    yspread = np.linspace(*yrng,100)
    xspread = np.linspace(*xrng,100)

    output = np.zeros((len(xspread),len(yspread)))
    output[:] = np.nan
    output2 = np.zeros((len(xspread),len(yspread)))
    output2[:] = np.nan    
    allsymbols = np.zeros((len(xspread),len(yspread)))
    allsymbols[:] = NONE

    for xdx,x in enumerate(xspread):
        for ydx,y in enumerate(yspread):
            try:
                ind = np.nanargmin((xax[indBases]-x)**2 + (yax[indBases]-y)**2)
            except ValueError:
                output2[xdx,ydx] = 0.3
                continue
            #print(x,y,zax[indBases][ind],data['symbol'][indBases][ind])
            if data['symbol'][indBases][ind]!=UPPER_IND and data['symbol'][indBases][ind]!=PISN_IND:
                output[xdx,ydx] = zax[indBases][ind]
            if data['symbol'][indBases][ind]==UPPER_IND:
                output2[xdx,ydx] = 0.3
                #print(data['base'][indBases][ind],data['initial_mass'][indBases][ind])
            
            allsymbols[xdx,ydx] = data['symbol'][indBases][ind]

    extent = (*xrng,*yrng)
    output = np.flipud(output.T)
    output2 = np.flipud(output2.T)

    ax.imshow(output2,extent=extent,interpolation='none',aspect='auto',cmap='Greys',vmin=0.0,vmax=1.0)
    

    im=ax.imshow(output,extent=extent,interpolation='none',aspect='auto',cmap=cm)
    cbar = fig.colorbar(im)
    
    
    
    if contours:
        allsymbols = allsymbols.T
        allsymbols[allsymbols==UPPER_IND] = PPISN_IND
        allsymbols[allsymbols==NONE] = 9999.9
        ax.contour(xspread,yspread,allsymbols,linewidths=3,colors='k',levels=1,extent=extent) #PISN line
        ax.contour(xspread,yspread,allsymbols,linewidths=3,colors='k',levels=[PPISN_IND],extent=extent) #PISN line
        #ax.contour(xspread,yspread,allsymbols,linewidths=3,linestyles='dashed',colors='k',extent=extent,levels=[NONE,PISN_IND,PPISN_IND,CC_IND],extend='neither') 
        if hatch == 1: # Hatch PISN
            ax.contourf(xspread,yspread, allsymbols, levels=[NONE,PISN_IND,PPISN_IND,CC_IND], extend='neither',linewidths=3,
                        cmap='gray',hatches=['\\\\',None,None,None],alpha=0.0)
        elif hatch == 2: #Hatch PPISN
            ax.contourf(xspread,yspread, allsymbols, levels=[NONE,PISN_IND,PPISN_IND,CC_IND], extend='neither',linewidths=3,
                        cmap='gray',hatches=[None,'\\\\',None,None],alpha=0.0)
        elif hatch == 3: #Hatch CC
            ax.contourf(xspread,yspread, allsymbols, levels=[NONE,PISN_IND,PPISN_IND,CC_IND], extend='neither',linewidths=3,
                        cmap='gray',hatches=[None,None,'\\\\',None],alpha=0.0)
                                             
    if labels:
        ax.text(0.8,0.4,'Pair Instability',fontsize=30, rotation=90, bbox=dict(facecolor='white', alpha=alpha), transform=ax.transAxes)
        ax.text(0.4,0.5,'Pulsations',fontsize=30, rotation=90, bbox=dict(facecolor='white', alpha=alpha), transform=ax.transAxes)
        ax.text(0.1,0.8,'Core Collapse',fontsize=30, rotation=90, bbox=dict(facecolor='white', alpha=alpha), transform=ax.transAxes)


    ax.tick_params(axis='x',which='minor',bottom=True)
    ax.tick_params(axis='y',which='minor',bottom=True)

    if xlabel is not None:
        ax.set_xlabel(xlabel,fontsize=fs, labelpad=10)
    else:
        ax.set_xlabel(r'$\rm{M}_{\rm{He}}\,[M_\odot]$',fontsize=fs, labelpad=10)

    if ylabel is not None:
        ax.set_ylabel(ylabel,fontsize=fs, labelpad=10)
    else:
        ax.set_ylabel(r'$\log_{10}\, Z$',fontsize=fs, labelpad=10)

    if xrng is not None:
        ax.set_xlim(xrng)

    if yrng is not None:
        ax.set_ylim(yrng)

    ax.yaxis.set_tick_params(labelsize=fs-2)
    
    ax.tick_params(axis='x',which='minor')
    ax.xaxis.set_tick_params(labelsize=fs-2)
        
    if zlabel is not None:
        cbar.set_label(zlabel,fontsize=fs, labelpad=10)
    else:
        cbar.set_label(r'$\rm{M}_{\rm{BH}}\,[M_\odot]$',fontsize=fs, labelpad=10)
    cbar.ax.tick_params(labelsize=fs-2)

    if len(name)>0:
        plt.savefig(fig_output+name,dpi=300)
    plt.show()

    return allsymbols


def plotMaxBH(data,bases,xaxis,yax,xrng=None,yrng=None,xlabel=None,ylabel=None,name='',loc=0,
             skip=[],ylog=False,xlog=False,ligo=False,bases2=None,ligo_bay=False,fs=20,line=False,
              fit=False,order=-1,y2=False,figsize=(12,9),error=None,errxy=None,r=4):
    
    if ligo: 
        fig, (ax, ax1) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[3, 1],'wspace':0.025, 'hspace':0.05},figsize=figsize)
    else:
        fig, ax = plt.subplots(1,1,figsize=figsize)
    
#     if y2:
#         ax2 = ax.twinx()
    
    fig.set_tight_layout(False)
    
    y2a=[]
    y2b=[]

    yaxp=[]
    yaxp2=[]
    for jdx,j in enumerate(bases):
        if dbg: print(j)
        if j in skip:
            continue

        ind = (data['base'] == j)

        if isinstance(yax,str): 
            yaxis = data[yax][ind]
        else:
            yaxis = yax[ind]
            
        bhmass = data['bhmass'][ind]
        
        symbol = data['symbol'][ind]

        if yaxis.dtype == 'U25':
            yaxis[np.where(yaxis=='')] = np.nan
            yaxis = yaxis.astype(np.float)

        if ylog:
            yaxis = np.log10(yaxis)
                        
        ind2 = ((symbol==PPISN_IND) | (symbol==CC_IND)) & (bhmass < 100.0)
        ind3 = ((symbol==PPISN_IND) | (symbol==CC_IND)) & (bhmass > 100.0)
        
        try:
            yaxp.append(np.nanmax(bhmass[ind2]))
        except ValueError:
            yaxp.append(0.0)
        try:
            yaxp2.append(np.nanmin(bhmass[ind3]))
        except ValueError:
            yaxp2.append(0.0)
        
        if y2:
            y2a.append(data['initial_mass'][ind][np.nanargmax(yaxis[ind2])])
            y2b.append(data['star_mass_prehydro'][ind][np.nanargmax(yaxis[ind2])])

        
    if xlog:
        xaxis=np.log10(xaxis)
        
    if isinstance(xaxis[0], numbers.Number):
        ax.scatter(xaxis,yaxp,c='k',s=sp+50)
        ax.scatter(xaxis,yaxp2,c='k',s=sp+50)
        if xrng is not None:
            ax.set_xlim(xrng)
        ax.xaxis.set_tick_params(labelsize=fs-2)
        ax.tick_params(axis='x',which='minor')
    else:
        xaxis = np.arange(0,len(yaxp))
        ax.scatter(xaxis,yaxp,c='k',s=sp+50)
        ax.scatter(xaxis,yaxp2,c='k',s=sp+50)
        ax.set_xticks(xaxis,xaxis)
        
    if line:
        ax.plot(xaxis,yaxp)
        ax.plot(xaxis,yaxp2)
    
    if fit:
        if order > 0:
            z = np.polyfit(xaxis,yaxp,order)
            p = np.poly1d(z)
            ax.plot(xaxis,p(xaxis),c='k',linewidth=5)
            bhy = p(xaxis)
            print(z)
            print([round(i,r) for i in z])
            p2=np.poly1d([round(i,r) for i in z])
            #ax.plot(xaxis,p2(xaxis),c='k',ls='--',linewidth=5)
            print(((p2(xaxis)-yaxp)/yaxp))
        else:
            for i in range(1,6):
                z = np.polyfit(xaxis,yaxp,i)
                p = np.poly1d(z)
                ax.plot(xaxis,p(xaxis),c='k',label=str(i))
            ax.legend(loc=0,fontsize=25)

    if yrng is not None:
        ax.set_ylim(yrng)
        
    
    if y2:
        print(y2a)
        ax.scatter(xaxis,y2a,c='r',s=sp-20)
        z = np.polyfit(xaxis,y2a,7)
        p = np.poly1d(z)
        windy = p(xaxis)
        
        ax.plot(xaxis,p(xaxis),'--',c='red',linewidth=2)
        
        print(y2b)
        ax.scatter(xaxis,y2b,c='blue',s=sp-20)
        z = np.polyfit(xaxis,y2b,7)
        p = np.poly1d(z)
        pulsey= p(xaxis)
        
        ax.plot(xaxis,p(xaxis),'--',c='blue',linewidth=2)
    
        ax.fill_between(xaxis,bhy,pulsey,facecolor='green',alpha=0.3,interpolate=True)
        ax.fill_between(xaxis,pulsey,windy,facecolor='yellow',alpha=0.5,interpolate=True) 
        
        ax.text(-2.85,58.0,'Winds',fontsize=30)
        ax.text(-4.75,49.0,'Pulses',fontsize=30)
        
        custom_lines = [Line2D([0], [0], color='red', lw=4,ls='--'),
                Line2D([0], [0], color='blue', lw=4,ls='--'),
                Line2D([0], [0], color='black', lw=4)]
        ax.legend(custom_lines, [r'$\rm{M_{He}}$',r'$\rm{M_{pre\, pulses}}$',r'$\rm{M_{BH,max}}$'],fontsize=30)
    
        ax.scatter(xaxis,y2b,c='blue',s=sp-20)
        ax.scatter(xaxis,y2a,c='r',s=sp-20)
        
        ax.fill_between(xaxis,0.0,bhy,facecolor='black',alpha=0.3,interpolate=True)
        ax.text(-4,20.0,'Black Holes',fontsize=30)
    

    if error is not None:
        ax.errorbar(errxy[0],errxy[1], yerr=error,c='black', capsize=17,linewidth=4,capthick=4)
    
    ax.tick_params(axis='y',which='minor')
    ax.yaxis.set_tick_params(labelsize=fs-2)

    if xlabel is None:
        ax.set_xlabel(r'$\rm{M}\,[M_\odot]$',fontsize=fs)
    else:
        ax.set_xlabel(xlabel,fontsize=fs)

    if ylabel is None:
        ax.set_ylabel(mbhl,fontsize=fs)
    else:
        ax.set_ylabel(ylabel,fontsize=fs)
    
    if ligo:
        overplot_ligo(fig,ax1,ax.get_ylim())
        
    if ligo_bay:
        plotLIGOGap(fig,ax)
    
    if len(name):
        plt.savefig(fig_output+name,dpi=300)
    plt.show()
    
    
def plotMaxBH2(data,bases,xaxis,yax,xrng=None,yrng=None,xlabel=None,ylabel=None,name='',loc=0,
             skip=[],ylog=False,xlog=False,ligo=False,bases2=None,ligo_bay=False,fs=20,line=False,show=True,
              fit=False,order=-1,y2=False,figsize=(12,9),error=None,errxy=None,r=4,fill=True,absgap=True,invalid=0.0,fig=None,ax=None):
    
    if ligo: 
        fig, (ax, ax1) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[3, 1],'wspace':0.025, 'hspace':0.05},figsize=figsize)
    else:
        if fig is None or ax is None:
            fig, ax = plt.subplots(1,1,figsize=figsize)
    
#     if y2:
#         ax2 = ax.twinx()
    
    fig.set_tight_layout(False)
    
    y2a=[]
    y2b=[]

    yaxp=[]
    yaxp2=[]
    for jdx,j in enumerate(bases):
        if dbg: print(j)
        if j in skip:
            continue

        ind = (data['base'] == j)

        if isinstance(yax,str): 
            yaxis = data[yax][ind]
        else:
            yaxis = yax[ind]
        
        symbol = data['symbol'][ind]
        bhmass = data['bhmass'][ind]

        if yaxis.dtype == 'U25':
            yaxis[np.where(yaxis=='')] = np.nan
        yaxis = yaxis.astype(np.float)

        if ylog:
            yaxis = np.log10(yaxis)
                        
        ind2 = ((symbol==PPISN_IND) | (symbol==CC_IND)) & (bhmass < 100.0)
        ind3 = ((symbol==PPISN_IND) | (symbol==CC_IND)) & (bhmass > 100.0)
        
        try:
            yaxp.append(yaxis[ind2][np.nanargmax(bhmass[ind2])])
        except ValueError:
            yaxp.append(invalid)
            
        try:
            yaxp2.append(yaxis[ind3][np.nanargmin(bhmass[ind3])])
        except ValueError:
            yaxp2.append(invalid)
            
        print(j,yaxp[-1],yaxp2[-1])
        
        y2a.append([j,data['intmass'][ind][np.nanargmax(bhmass[ind2])]])
        y2b.append([j,data['intmass'][ind][np.nanargmin(bhmass[ind3])]])
      #  print(j,yaxis)
        
       # try:
       #     print(j,data['intmass'][ind][np.nanargmax(yaxis[ind2])],yaxp[-1])
       # except:
       #     print(j,data['intmass'][ind],yaxp[-1])
        
       # try:
       #     print(j,data['intmass'][ind][np.nanargmin(yaxis[ind3])],yaxp2[-1])
      #  except:
      #      print(j,data['intmass'][ind],yaxp2[-1])
        
      #  print()
        
    if xlog:
        xaxis=np.log10(xaxis)
        
    
        
    if isinstance(xaxis[0], numbers.Number):
        ax.scatter(xaxis,yaxp,c='k',s=sp+50)
        ax.scatter(xaxis,yaxp2,c='k',s=sp+50)
        ax.xaxis.set_tick_params(labelsize=fs-2)
        ax.tick_params(axis='x',which='minor')
    else:
        xaxis = np.arange(0,len(yaxp))
        ax.scatter(xaxis,yaxp,c='k',s=sp+50)
        ax.scatter(xaxis,yaxp2,c='k',s=sp+50)
        ax.set_xticks(xaxis,xaxis)
        
    print(np.array(yaxp2)-np.array(yaxp))
    print(','.join(map(str, xaxis)) )
    print(','.join(map(str, yaxp2)) )
    print(','.join(map(str, yaxp2)) )
        
    if line:
        ax.plot(xaxis,yaxp)
        ax.plot(xaxis,yaxp2)
    
    if fit:
        for yy in [yaxp,yaxp2]:
            if order > 0:
                z = np.polyfit(xaxis,yy,order)
                p = np.poly1d(z)
                ax.plot(xaxis,p(xaxis),c='k',linewidth=5)
                bhy = p(xaxis)
                print(z)
                print([round(i,r) for i in z])
                p2=np.poly1d([round(i,r) for i in z])
                #ax.plot(xaxis,p2(xaxis),c='k',ls='--',linewidth=5)
                print(((p2(xaxis)-yy)/yy))
            else:
                for i in range(1,6):
                    z = np.polyfit(xaxis,yy,i)
                    p = np.poly1d(z)
                    ax.plot(xaxis,p(xaxis),c='k',label=str(i))
                ax.legend(loc=0,fontsize=25)

    if yrng is not None:
        ax.set_ylim(yrng)
        
    if xrng is not None:
        ax.set_xlim(xrng)
        
    if fill:
        ax.fill_between(xaxis,[0.0]*len(yaxp),yaxp,facecolor='blue',alpha=0.3,interpolate=True)
        ax.fill_between(xaxis,yaxp2,[200]*len(yaxp2),facecolor='orange',alpha=0.5,interpolate=True) 
        
        ax.text(-2.3,40.0,'BHs',fontsize=30)
        ax.text(1.0,160.0,'BHs',fontsize=30)
        ax.text(-2.5,130,'Mass Gap',fontsize=30)
        ax.text(1.2,60,'Mass Gap',fontsize=30)
        
    if absgap:
        ax.fill_between([np.min(xaxis),np.max(xaxis)],np.max(yaxp),np.min(yaxp2),facecolor='black',alpha=0.5)
        #ax.text(-0.5,105.0,'Ruled Out',fontsize=30)
    

    if error is not None:
        ax.errorbar(errxy[0],errxy[1], yerr=error,c='black', capsize=17,linewidth=4,capthick=4)
    
    ax.tick_params(axis='y',which='minor')
    ax.yaxis.set_tick_params(labelsize=fs-2)
    
    ax.set_xticks([-3,-2,-1,0,1,2,3])

    if xlabel is None:
        ax.set_xlabel(r'$\rm{M}\,[M_\odot]$',fontsize=fs)
    else:
        ax.set_xlabel(xlabel,fontsize=fs)

    if ylabel is None:
        ax.set_ylabel(mbhl,fontsize=fs)
    else:
        ax.set_ylabel(ylabel,fontsize=fs)
    
    if ligo:
        overplot_ligo(fig,ax1,ax.get_ylim())
        lim=ax1.get_xlim()
        ax1.fill_between(lim,40.8-4.4,40.8+11.8,color='red',alpha=0.5)
        ax1.set_xlim(lim)
        
    if ligo_bay:
        plotLIGOGap(fig,ax)
    
    if len(name):
        plt.savefig(fig_output+name,dpi=300)
        
    if show:
        plt.show()
    
def plotMaxBHMulti(data,list_bases,list_xaxis,yax,xrng=None,yrng=None,xlabel=None,ylabel=None,name='',
             skip=[],ylog=False,xlog=False,ligo=False,list_labels=[],fs=30,c1=None,c2=None,c3=None):
    
    if ligo:
        fig, (ax, ax1) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[3, 1],'wspace':0.025, 'hspace':0.05})
    else:
        fig, ax = plt.subplots(1,1)
    
    fig.set_tight_layout(False)

    for idb,bases in enumerate(list_bases):
        yaxp=[]
        for jdx,j in enumerate(bases):
            print(j)
            if j in skip:
                continue

            ind = (data['base'] == j)

            if isinstance(yax,str): 
                yaxis = data[yax][ind]
            else:
                yaxis = yax[ind]

            symbol = data['symbol'][ind]

            if yaxis.dtype == 'U25':
                yaxis[np.where(yaxis=='')] = np.nan
                yaxis = yaxis.astype(np.float)

            if ylog:
                yaxis = np.log10(yaxis)

            ind = (symbol==PPISN_IND) | (symbol==CC_IND)

            try:
                yy = np.nanmax(yaxis[ind])
            except ValueError:
                yy = -1
            
            yaxp.append(yy)

        xaxis=list_xaxis[idb]
        if xlog:
            xaxis=np.log10(xaxis)
        yaxp=np.array(yaxp)
        indbh=yaxp>0.0
        if isinstance(xaxis[0], numbers.Number):
            #print(xaxis,yaxp)
            l,=ax.plot(xaxis[indbh],yaxp[indbh],label=list_labels[idb])
            ax.scatter(xaxis,yaxp,c=l.get_color(),s=sp)
            
            if xrng is not None:
                ax.set_xlim(xrng)
            ax.xaxis.set_tick_params(labelsize=fs-2)
            ax.tick_params(axis='x',which='minor')
        else:
            xx = np.arange(0,len(yaxp))
            l,=ax.plot(xaxis[indbh],yaxp[indbh],label=list_labels[idb])
            ax.scatter(xx,yaxp,c=l.get_color(),s=sp)
            ax.set_xticks(xx,xaxis)

    if yrng is not None:
        ax.set_ylim(yrng)
    
    
    ax.tick_params(axis='y',which='minor')

    #fs = 20
    if xlabel is None:
        ax.set_xlabel(r'$\rm{M}\,[M_\odot]$',fontsize=fs)
    else:
        ax.set_xlabel(xlabel,fontsize=fs)

    if ylabel is None:
        ax.set_ylabel(mbhl,fontsize=fs)
    else:
        ax.set_ylabel(ylabel,fontsize=fs)

    ax.yaxis.set_tick_params(labelsize=fs)
    
    ax.legend(loc=0,fontsize=fs)
    
    if ligo:
        overplot_ligo(fig,ax1,ax.get_ylim())
    
    if len(name):
        plt.savefig(fig_output+name,dpi=300)
    plt.show()
    
    
def plotMaxBHMultiP(data,list_bases,list_xaxis,yax='bhmass',xrng=None,xlabel=None,ylabel=None,name='',
             skip=[],ylog=False,ligo=False,fs=20,pad=15,box=True,c1=None,c2=None,c3=None,calpha=0.25,figsize=(15,9)):
    
    if ligo:
        fig, (ax, ax1) = plt.subplots(1,2, gridspec_kw = {'width_ratios':[3, 1],'wspace':0.025, 'hspace':0.05},figsize=figsize)
    else:
        fig, ax = plt.subplots(1,1,figsize=figsize)
    
    fig.set_tight_layout(False)
        

    for idb,bases in enumerate(list_bases[::-1]):
        yval = []
        for jdx,j in enumerate(bases):
            if dbg: print(j)
            if j in skip:
                continue

            ind = (data['base'] == j)

            if isinstance(yax,str): 
                yaxis = data[yax][ind]
            else:
                yaxis = yax[ind]

            if yaxis.dtype == 'U25':
                yaxis[np.where(yaxis=='')] = np.nan
                yaxis = yaxis.astype(np.float)

            if ylog:
                yaxis = np.log10(yaxis)

            symbol = data['symbol'][ind]
            ind = (symbol==PPISN_IND)&(np.logical_not(data['failed'][ind]))

            try:
                yval.append(np.nanmax(yaxis[ind]))
            except ValueError:
                yval.append(np.nan)
        
        if dbg: print(yval)
        maxy=np.nanmax(yval)
        miny=np.nanmin(yval)
        mid=(maxy+miny)/2.0
                
        if dbg: print(idb,mid,maxy,miny)
        ax.errorbar(mid,idb, xerr=np.array([[mid-miny,maxy-mid]]).T,c='black', capsize=17,linewidth=4,capthick=4)
         
            
    ax.set_ylim(-0.5,len(list_bases)-0.5)
    ax.tick_params(axis='y', pad=pad)
            
    ax.set_yticks(range(len(list_bases)))
    ax.set_yticklabels(list_xaxis[::-1],fontsize=fs-2)
            
    if xrng is not None:
        ax.set_xlim(xrng)
    
    
    ax.tick_params(axis='y', which='minor', left=False,right=False)
    ax.tick_params(axis='x',which='minor')
    
    if box:
        xmin,xmax = ax.get_xlim()
        ymin,ymax = ax.get_ylim()
        b1 = 1.0
        b2 = 7.0
        
        plotBox(ax,xmin,xmax,ymax-b1,ymax,c1,alpha=calpha)
        plotBox(ax,xmin,xmax,ymax-b2,ymax-b1,c2,alpha=calpha)
        plotBox(ax,xmin,xmax,ymin,ymax-b2,c3,alpha=calpha)
        
        xx=49
        ax.text(xx+1,0.6,'Numerics',fontsize=40)
        ax.text(xx+1.5,5.25,'Physics',fontsize=40)
        ax.text(xx,8.75,'Environment',fontsize=40)

    #fs = 20
    #ax.set_xlabel('Parameter',fontsize=fs)

    if xlabel is None:
        ax.set_xlabel(mbhl,fontsize=fs)
    else:
        ax.set_xlabel(xlabel,fontsize=fs)

    ax.xaxis.set_tick_params(labelsize=fs)
        
    if len(name):
        plt.savefig(fig_output+name,dpi=300)
    plt.show()

def plotLIGOGap(fig,ax):
    
    amin=-4.5
    amed=41.6    
    amax=9.0
    
    bmin=-5.7
    bmed=42.0
    bmax=15.0

    xlim = ax.get_xlim()
    
    acol = 'blue'
    bcol = 'orange'
    alpha = 0.25
    
    plotBox(ax,*xlim,amed+amin,amed+amax,acol,alpha)
    ax.plot([*xlim],[amed,amed],c=acol,linewidth=3) 
    ax.plot([*xlim],[amed+amin,amed+amin],'--',c=acol,linewidth=3)
    ax.plot([*xlim],[amed+amax,amed+amax],'--',c=acol,linewidth=3)
    
    #plotBox(ax,*xlim,bmed+bmin,bmed+bmax,bcol,alpha)
    #ax.plot([*xlim],[bmed,bmed],c=bcol,linewidth=3)
    #ax.plot([*xlim],[bmed+bmin,bmed+bmin],'--',c=bcol,linewidth=3)
    #ax.plot([*xlim],[bmed+bmax,bmed+bmax],'--',c=bcol,linewidth=3)
    
@contextlib.contextmanager
def smart_open(filename=None):
    if filename:
        fh = open(filename, 'w')
    else:
        fh = sys.stdout

    try:
        yield fh
    finally:
        if fh is not sys.stdout:
            fh.close()
    
def table(data,bases,param,value,skip=[],
         output=None,
         header=None,
         latex=True,filename=None,fmt=''):
    
    sep = ' '
    endline = ''
    if latex:
        sep = " & "
        endline = ' \\\\'
        
    with smart_open(filename) as fh:        
        print(sep.join(header),file=fh)    

        for jdx,j in enumerate(bases):
            if j in skip:
                continue

            ind = (data['base'] == j)    

            for ldx,line in enumerate(data[ind]):
                if line['symbol']==PISN_IND:
                    ty = 'PISN'
                    bhmass = 0.0
                elif line['symbol']==PPISN_IND:
                    ty = 'PPISN'
                elif line['symbol']==CC_IND:
                    ty = 'CC'
                else:
                    continue
                    
                out2 = []
                for y in output:
                    if isinstance(y[ind][ldx],np.ndarray):
                        out2.append(y[ind][ldx][0])
                    else:
                        out2.append(y[ind][ldx])
                    
                x = [fmt.format(y) for y in out2]
                    
                print(sep.join([param[jdx],value[jdx],*x,ty]),endline,file=fh)

        

def fixsc12ah():
    import pickle
    for i,j in zip([40,42,44,46,48,50,52,54],[28.88,30.36,30.4,32.92,34.23,35.37,36.76,37.87]):
        with open(str(i)+'/dump.txt','rb') as f:
            x=pickle.load(f)
        x['comass_pre']=j
        with open(str(i)+'/dump.txt','wb') as f:
            pickle.dump(x,f)


def fitBHmass(data,bases,values,fitfunc,minv=38.0,maxv=60.0,xax='comass_pre',yax='bhmass',skip=[],
            plot=True,testv=[],xlog=False,ylog=False,r=4):
    
    x=[]
    y=[]
    z=[]
    for jdx,j in enumerate(bases):
        print(j)
        if j in skip:
            continue

        ind = (data['base'] == j)

        if isinstance(xax,str): 
            xaxis = data[xax][ind]
        else:
            xaxis = xax[ind]

        if isinstance(yax,str): 
            yaxis = data[yax][ind]
        else:
            yaxis = yax[ind]

        if yaxis.dtype == 'U25':
            yaxis[np.where(yaxis=='')] = np.nan
            yaxis = yaxis.astype(np.float)
        if xaxis.dtype == 'U25':
            xaxis[np.where(xaxis=='')] = np.nan  
            xaxis = xaxis.astype(np.float)

        if xlog:
            xaxis = np.log10(xaxis)

        if ylog:
            yaxis = np.log10(yaxis)

        symbol = data['symbol'][ind]

        ind2 = np.logical_and(np.logical_or(symbol==PPISN_IND,symbol==CC_IND),np.logical_and(xaxis>=minv,xaxis<=maxv))
        
        #print(np.count_nonzero(ind),np.count_nonzero(ind2),np.count_nozero(np.logical_or(symbol==PPISN_IND,symbol==CC_IND)),np.count_nonzero())
        
        x.append(xaxis[ind2])
        z.append(yaxis[ind2])
        y.append([values[jdx]]*np.count_nonzero(ind2))
    
    x=np.concatenate(x).ravel()
    y=np.concatenate(y).ravel()
    z=np.concatenate(z).ravel()
    
    f,_=scipy.optimize.curve_fit(fitfunc, (x,y), z,bounds=(-500,500))
    print(f)
    f = [round(i,4) for i in f]
    print(f)   
    if plot:
        fig=plt.figure(figsize=(16,9))
        ax=fig.add_subplot(111)
        plt.scatter(x,z,c=y,s=20,cmap='tab10')
        cbar=plt.colorbar()
        ticklabs = cbar.ax.get_yticklabels()
        cbar.ax.set_yticklabels(ticklabs, fontsize=30)
        xx = np.linspace(minv,maxv,100)
        for i in testv:
            zzz = fitfunc(np.array((xx,[i]*len(xx))), *f)
            ax.plot(xx, zzz,label=str(i),linewidth=2)
        
        for j in values:
            ind=(y==j)
            r=[]
            for i,k in zip(x[ind],z[ind]):
                r.append((fitfunc([i,10**j],*f)-z)/z)
            print(j,np.median(np.abs(r)),np.mean(np.abs(r)))
        plt.legend(loc=0,fontsize=30)
        ax.xaxis.set_tick_params(labelsize=28)
        ax.yaxis.set_tick_params(labelsize=28)
        plt.show()

        
        
def getRateNames(folder):
    rates = []
    with open(os.path.join(folder,'rate_tables','rates_list.txt'),'r') as f:
        for idx,i in enumerate(f):
            rates.append({})
            rates[-1]['name'] = i.split()[-1]
            rates[-1]['iso'] = rates[-1]['name'].split('_')[1]
            _,rates[-1]['p'], rates[-1]['n'] = mpplot._getIso(rates[-1]['iso'])
    return rates




def plot2d2(data,bases,xax,yax,zax,xlabel=None,ylabel=None,zlabel=None,xlog=False,ylog=False,zlog=False,
           xrng=None,yrng=None,labels=False,fs=30,figsize=(12,9),lpad=5,cm='plasma',s=20,skip=[],zrng=[0,1],fig=None,ax=None):
    
    if fig is None:
        fig=plt.figure(figsize=figsize)
        ax=fig.add_subplot(111)
    #ax.set_yscale("log", nonposy='clip')

    for j,x in zip(bases,xax):
        if dbg: print(j)
        if j in skip: 
            continue

        ind = (data['base'] == j)

        if isinstance(yax,str): 
            yaxis = data[yax][ind]
        else:
            yaxis = yax[ind]
        
        symbol = data['symbol'][ind]

        if yaxis.dtype == 'U25':
            yaxis[np.where(yaxis=='')] = np.nan
            yaxis = yaxis.astype(np.float)

        if ylog:
            yaxis = np.log10(yaxis)
                        
        ####
        if isinstance(zax,str): 
            zaxis = data[zax][ind]
        else:
            zaxis = zax[ind]
        
        symbol = data['symbol'][ind]

        if zaxis.dtype == 'U25':
            zaxis[np.where(zaxis=='')] = np.nan
            zaxis = zaxis.astype(np.float)

        if zlog:
            zaxis = np.log10(zaxis)
                
        xx = [x]*len(yaxis)
        im = ax.scatter(xx,yaxis,c=zaxis,s=s,cmap=cm,vmin=zrng[0],vmax=zrng[1])
                
    plt.colorbar(im)

    if yrng is not None:
        ax.set_ylim(yrng)
        
    if xrng is not None:
        ax.set_xlim(xrng)
        
    plt.show()
    
    return 



def plot2d3(data,bases,xax,yax,zax,xlabel=None,ylabel=None,zlabel=None,xlog=False,ylog=False,zlog=False,xnum=0,ynum=0,masses=None,
           xrng=None,yrng=None,labels=False,fs=30,figsize=(12,9),lpad=5,cm='plasma',s=20,skip=[],zrng=[0,1],fig=None,ax=None,cbar_label='',invalid=0.5):
    
    if fig is None:
        fig=plt.figure(figsize=figsize)
        ax=fig.add_subplot(111)
    #ax.set_yscale("log", nonposy='clip')

    box = np.zeros((len(masses),len(xax)))
    box[:,:] = np.nan
    box2 = np.zeros((len(masses),len(xax)))
    box2[:,:] = 0
    
    for jdx,j in enumerate(bases):
        for idx,i in enumerate(masses):
            ind = (data['base'] == j) & (data[yax] == i)
            
            if np.count_nonzero(ind) == 0:
                box[idx,jdx] = np.nan
                box2[idx,jdx] = invalid
                continue
            zaxis = zax[ind][0]
            if zlog:
                zaxis = np.log10(zaxis)
            if idx==0:
                print(jdx,idx,zaxis)
            box[idx,jdx] = zaxis

            
    ax.imshow(box2,extent=(np.min(xax)-0.25,np.max(xax)+0.25,np.min(masses),np.max(masses)),aspect='auto',origin='lower',interpolation='none',cmap='Greys',vmin=0.0,vmax=1.0)
    im=ax.imshow(box,extent=(np.min(xax)-0.25,np.max(xax)+0.25,np.min(masses),np.max(masses)),aspect='auto',origin='lower',interpolation='none',norm=matplotlib.colors.LogNorm())

    cbar = plt.colorbar(im)
    cbar.set_label(cbar_label,fontsize=30)
    ax.set_xticks([-3,-2,-1,0,1,2,3])
    
        
    #plt.show()
    
    return 


def plotLineFixedZ(data,bases,xaxis,yax,zax,zvalue,xrng=None,yrng=None,xlabel=None,ylabel=None,name='',loc=0,
             skip=[],ylog=False,xlog=False,ligo=False,bases2=None,ligo_bay=False,fs=20,line=False,show=True,lc='k',
              fit=False,order=-1,y2=False,figsize=(12,9),error=None,errxy=None,r=4,fill=True,absgap=True,invalid=0.0,fig=None,ax=None,high=False):
    
    if fig is None or ax is None:
        fig, ax = plt.subplots(1,1,figsize=figsize)
    
#     if y2:
#         ax2 = ax.twinx()
    
    fig.set_tight_layout(False)
    
    y2a=[]
    y2b=[]
    
    if ylog:
        ax.set_yscale('log')

    yaxp=[]
    yaxp2=[]
    symbol = []
    for jdx,j in enumerate(bases):
        if dbg: print(j)
        if j in skip:
            continue

            
        ind = (data['base'] == j)&(data[zax]==zvalue)
        if np.count_nonzero(ind)==0:
            ind = (data['base'] == j)&(data[zax]==str(round(float(zvalue),1)))
        
        if isinstance(yax,str): 
            yaxis = data[yax][ind]
        else:
            yaxis = yax[ind]
        
        print(j,np.count_nonzero(ind),yaxis)
        if np.count_nonzero(ind):
            yaxp.append(yaxis[0])
            symbol.append(data['symbol'][ind][0])
        else:
            yaxp.append(0.0)
            symbol.append(UPPER_IND)
        
    yaxp = np.array(yaxp).astype(float)
    symbol = np.array(symbol).astype(int)
    
   # if ylog:
   #     yaxp = np.log10(yaxp)
    
    if xlog:
        xaxis=np.log10(xaxis)
    
                
    if isinstance(xaxis[0], numbers.Number):
        if xrng is not None:
            ax.set_xlim(xrng)
        ax.xaxis.set_tick_params(labelsize=fs-2)
        ax.tick_params(axis='x',which='minor')
    else:
        xaxis = np.arange(0,len(yaxp))
        ax.set_xticks(xaxis,xaxis)
    
    print(xaxis)
    xaxis = np.array([float(i) for i in xaxis])
    
    ind = (symbol == PISN_IND)
    if np.count_nonzero(ind) >0:
        ax.scatter(xaxis[ind],yaxp[ind],c='red',s=sp+50,marker='*')

    ind = (symbol == PPISN_IND)
    print(ind)
    if np.count_nonzero(ind) >0:
        ax.scatter(xaxis[ind],yaxp[ind],c='green',s=sp+50,marker='s')
    
    ind = (symbol == CC_IND)
    print(ind)
    if np.count_nonzero(ind) >0:
        if high:
            ax.scatter(xaxis[ind],yaxp[ind],c='yellow',s=sp+50,marker="o")
        else:
            ax.scatter(xaxis[ind],yaxp[ind],c='blue',s=sp+50,marker="o")
        
    if line:
        ax.plot(xaxis,yaxp,c='black',linewidth=2)
    

    if yrng is not None:
        ax.set_ylim(yrng)
            
    ax.tick_params(axis='y',which='minor')
    ax.yaxis.set_tick_params(labelsize=fs-2)

    if xlabel is None:
        ax.set_xlabel(r'$\rm{M}\,[M_\odot]$',fontsize=fs)
    else:
        ax.set_xlabel(xlabel,fontsize=fs)

    if ylabel is None:
        ax.set_ylabel(mbhl,fontsize=fs)
    else:
        ax.set_ylabel(ylabel,fontsize=fs)
        
    if len(name):
        plt.savefig(fig_output+name,dpi=300)
      
    if show:
        plt.show()
        
        
def getMaxBHMasses(data,bases,skip=[],bad=[],invalid=np.nan):
    

    y2a=[]
    y2b=[]

    yaxp=[]
    yaxp2=[]
    edge=[]
    for jdx,j in enumerate(bases):
        if dbg: print(j)
        if j in skip:
            continue
        
        ind = (data['base'] == j)

        for i in bad:
            if i[0]==j:
                ind = ind & np.logical_not(data['intmass']==int(i[1]))
                
        symbol = data['symbol'][ind]
        bhmass = data['bhmass'][ind]
        intmass = data['intmass'][ind].astype('float')
        yaxis = data['bhmass'][ind]
        
        
       # print(j,intmass)
        
        ind2 = ((symbol==PPISN_IND) | (symbol==CC_IND)) & (bhmass < 100.0)
        ind3 = ((symbol==PPISN_IND) | (symbol==CC_IND)) & (bhmass > 100.0)

        try:
            yaxp.append(yaxis[ind2][np.nanargmax(bhmass[ind2])])
        except ValueError:
            yaxp.append(invalid)
            
        try:
            yaxp2.append(yaxis[ind3][np.nanargmin(bhmass[ind3])])
        except ValueError:
            yaxp2.append(invalid)
        
        y2a.append(intmass[ind2][np.nanargmax(bhmass[ind2])])
        y2b.append(intmass[ind3][np.nanargmin(bhmass[ind3])])
        
        
        #Edge between CC and PPISN
        zzz=(symbol==CC_IND) & (bhmass < 100.0)
        indy = np.nanmax(intmass[zzz])
        #print(j,intmass[zzz],indy)
        edge.append(indy)
        
        
    return {'lower_BH':yaxp,'upper_BH':yaxp2,'lower_int':y2a,'upper_int':y2b,'edge':edge}
    
       
