# pisn_rates.py needs to be in PYTHONPATH

import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import mesaPlot as mp
import os
import sys
import warnings
import glob
import itertools
import h5py
import scipy.stats

#pip3 install matplotlib-label-lines --user
from labellines import labelLine, labelLines

warnings.filterwarnings("ignore")

sys.path.append('../')

import pisn_rates as pisn

p=mp.plot()


plt.style.use('mesa.mplstyle')
output='./'

#masses=np.array(np.arange(30,200,2))

bases_c12_rate = ['m30','m25','m20','m15','m10','m05','p00','p05','p10','p15','p20','p25','p30']
labelsc12_rate = ['-3','-2.5','-2.0','-1.5','-1.0','-0.5','0.0','0.5','1.0','1.5','2.0','2.5','3.0']

bases_others = ['cf88','deboer','deboer_m','deboer_p','jina','kunz','nacre']
labels_others = bases_others

bases_high = ['3alow1s_h','3ahigh1s_h','o16high_h','o16low_h','tum_h_h','tum_l_h','tum_m_h']
labels_high = bases_high

bases_low = ['3alow1s_l','3ahigh1s_l','o16high_l','o16low_l','tum_h_l','tum_l_l','tum_m_l']
labels_low = bases_low

bases_mid = ['3alow1s_m','3ahigh1s_m','o16high_m','o16low_m','tum_h_m','tum_l_m','tum_m_m']
labels_mid = bases_mid


bases_split = ['corehe','shellhe','exphe']
labels_split = bases_split


masses = np.arange(30,201,1)
masses = np.array([round(i,1) for i in masses])

G=6.67428*10**-8
msun=1.9892*10**33
rsun=6.9598*10**10

data = np.genfromtxt('../../data/all_models.txt',names=True,dtype=None,encoding=None) 


fig=plt.figure(figsize=(12,9))
ax=fig.add_subplot(111)

pisn.plot2d3(data,bases_c12_rate,[float(i) for i in labelsc12_rate],'intmass',data['center_c12_posthe'],labelsc12_rate,skip=[],zlog=False,yrng=[30,200],s=600,cm=None,
            masses=masses,zrng=[-3.0,0.3],xnum=len(labelsc12_rate),ynum=171,fig=fig,ax=ax,cbar_label=r'$\rm{X\left(^{12}C\right)}$',invalid=0.5)

ax.set_ylabel(r'$\rm{M}_{\rm{He,Initial}}\,[M_\odot]$',fontsize=30)
ax.set_xlabel(r'$\sigma_{\rm{C12}}$',fontsize=30)


res = pisn.getMaxBHMasses(data,bases_c12_rate,bad=[['m20',71],['m20',72],['m15','57'],['m05',47],['p00',45],['p00',46],['p05',38],['p15',41],['p15',43],['p20','41'],['p25',39]])


cctopregion=res['upper_int']
ppisnlower=[97,90,84,66,55,50,45,45,41,38,38,38,38]

ppisnupper=[97,90,84,78,73,68,65,63,61,60,60,60,60]

# print(cctopregion,len(cctopregion))
# print(ppisnupper,len(ppisnupper))
# print(ppisnlower,len(ppisnlower))

sigmas=[float(i) for i in labelsc12_rate]

ax.plot(sigmas,cctopregion,c='white',linewidth=2)
ax.plot(sigmas,ppisnlower,c='white',linewidth=2)
ax.plot(sigmas,ppisnupper,c='white',linewidth=2)


ax.text(1.5,46.0,'PPISN',fontsize=30,color='black')
ax.text(-0.5,105,'PISN',fontsize=30,color='black')
ax.text(-2.5,50,'CC',fontsize=30,color='black')            
ax.text(-2.5,175,'CC',fontsize=30,color='black')

ax.set_xlim(-3.0,3.0)
fig.savefig('fig2.pdf',dpi=300)