# pisn_rates.py needs to be in PYTHONPATH

import numpy as np
import matplotlib.pyplot as plt
import matplotlib as mpl
import matplotlib
import mesaPlot as mp
import os
import sys
import warnings
import glob
import itertools
import h5py
import scipy.stats

#pip3 install matplotlib-label-lines --user
from labellines import labelLine, labelLines

warnings.filterwarnings("ignore")

sys.path.append('../')

import pisn_rates as pisn

p=mp.plot()


plt.style.use('mesa.mplstyle')
output='./'

#masses=np.array(np.arange(30,200,2))

bases_c12_rate = ['m30','m25','m20','m15','m10','m05','p00','p05','p10','p15','p20','p25','p30']
labelsc12_rate = ['-3','-2.5','-2.0','-1.5','-1.0','-0.5','0.0','0.5','1.0','1.5','2.0','2.5','3.0']

bases_others = ['cf88','deboer','deboer_m','deboer_p','jina','kunz','nacre']
labels_others = bases_others

bases_high = ['3alow1s_h','3ahigh1s_h','o16high_h','o16low_h','tum_h_h','tum_l_h','tum_m_h']
labels_high = bases_high

bases_low = ['3alow1s_l','3ahigh1s_l','o16high_l','o16low_l','tum_h_l','tum_l_l','tum_m_l']
labels_low = bases_low

bases_mid = ['3alow1s_m','3ahigh1s_m','o16high_m','o16low_m','tum_h_m','tum_l_m','tum_m_m']
labels_mid = bases_mid


bases_split = ['corehe','shellhe','exphe']
labels_split = bases_split


masses = np.arange(30,201,1)
masses = np.array([round(i,1) for i in masses])

G=6.67428*10**-8
msun=1.9892*10**33
rsun=6.9598*10**10

data = np.genfromtxt('../../data/all_models.txt',names=True,dtype=None,encoding=None) 

####################################################3


cpad=15.0
yaxis=r"$m\, [M_{\odot}]$"
xlabel=r"$\rm{Model \ Number}$"

cbar_label=r'$\int_0^m \Gamma_1 P/\rho\, \rm{d}m  -4/3$'
cbar_eng=r'$\rm{signed\ \log\,\left(|\epsilon|\right)}$'

cmap='RdBu'

fig, axs =plt.subplots(ncols=3,nrows=4,figsize=(36,28),gridspec_kw={'width_ratios':[1,1,1],'height_ratios':[1,9,9,9]})
p=mp.plot()


l1=['A','B','C','E']
l2=['A','B','D','E']
l3=['A','D','E']

m1=[2000,2400,3300,4500]
t1=[r'$1\,\rm{yr}$',r'$1\,\rm{yr}$',r'$1\,\rm{month}$']

m2=[2400,2700,3000,3600]
t2=[r'$1\,\rm{yr}$',r'$1\,\rm{day}$',r'$ 1\,\rm{min}$']

m3=[2500,2800,3700]
t3=[r'$10\,\rm{days}$',r'$1\,\rm{min}$']


xrng1=[2000,5000]
xrng2=[2400,4200]
xrng3=[2500,4200]

def turnoff(fig,ax,label,xrng,frac,mm,tt,tfrac):
    ax.grid(False)
    ax.set_xlim(*xrng)
    ax.set_ylim(0,1)
    
    ax.annotate(label,xy=frac,xycoords='axes fraction',fontsize=36,color='black')
    # Hide axes ticks
    ax.set_xticks([])
    ax.set_yticks([])
    cmap2 = mpl.colors.ListedColormap(['white'])
    cb=fig.colorbar(plt.cm.ScalarMappable(norm=plt.Normalize(vmin=0, vmax=1), cmap=cmap2), ax=ax,ticks=[])
    cb.outline.set_edgecolor('white')
    cb.set_label('Time',color='white')
    ax.spines['bottom'].set_color('white')
    ax.spines['top'].set_color('white') 
    ax.spines['right'].set_color('white')
    ax.spines['left'].set_color('white')

    for ms,me,t,tf in zip(mm,mm[1:],tt,tfrac):
        astart,aend=ax.get_xlim()
        
        lstart=((ms-astart)/(aend-astart)) 
        lend=((me-astart)/(aend-astart))         
        ax.annotate('', xy=(lend,-1), xytext=(lstart,-1),xycoords='axes fraction', arrowprops={'arrowstyle':'|-|','linewidth':2})
        
        mid = lstart +(lend-lstart)/tf
        ax.annotate(t,xy=(mid,0),xycoords='axes fraction')       
        #ax.annotate(tt)
        #ax.annotate('', xy=(me,0), xytext=(ms,0), arrowprops={'arrowstyle':'|-|','linewidth':2})
        #mid = ms +(me-ms)/tf
        #ax.annotate(t,xy=(mid,0.1))    
        
        
ax = axs[0,0]
turnoff(fig,ax,r"$\rm{Core \ Collapse}\ \left(\sigma_{\rm{C12}}=-3.0\right)$",xrng1,(0.15,1.0),m1,t1,(4,2.5,4))

ax = axs[0,1]
turnoff(fig,ax,r"$\rm{PPISN}\ \left(\sigma_{\rm{C12}}=0.0\right)$",xrng2,(0.2,1.0),m2,t2,(3,7,3))

ax = axs[0,2]
turnoff(fig,ax,r"$\rm{PISN}\ \left(\sigma_{\rm{C12}}=3.0\right)$",xrng3,(0.2,1.0),m3,t3,(10,2.5))



##############################
ax = axs[1,0]
m=mp.MESA()
m.log_fold='../../data/kips/m30/60/LOGS'
m.loadHistory()

p.plotKip3(m,xaxis='model_number',mod_min=xrng1[0],mod_max=xrng1[1],colorbar=True,show_mix_labels=False,
           mix_alpha=0.8,cmin=-18,cmax=18,fig=fig,ax=ax,show=False,yrng=[0.0,60.0],cbar_label=cbar_eng,
            mix_hatch=True,dbg=True,cpad=cpad)

t=ax.annotate('Oxygen', xy=(4800,2),  xycoords='data',color='black',fontsize=30,
            xytext=(4600,10), textcoords='data',
            arrowprops=dict(facecolor='white', shrink=0.05),
            horizontalalignment='right', verticalalignment='top',
            )
t.set_bbox(dict(facecolor='white', alpha=0.5))

t=ax.text(3000,48.0,'Helium',color='black',fontsize=30)

t.set_bbox(dict(facecolor='white', alpha=0.5))


t=ax.annotate('Carbon', xy=(2700,6),  xycoords='data',color='black',fontsize=30,
            xytext=(2700,15), textcoords='data',
            arrowprops=dict(facecolor='white', shrink=0.05),
            horizontalalignment='right', verticalalignment='top',
            )

t.set_bbox(dict(facecolor='white', alpha=0.5))


ax.set_xlabel(xlabel,labelpad=1)
ax.set_ylabel(yaxis)


ax2=ax.twiny()
ax2.set_xlim(*ax.get_xlim())
ax2.xaxis.set_ticks_position('none')
ax2.xaxis.set_label_position('top')
ax2.set_xticks(m1)
ax2.set_xticklabels(l1)


for i,j in zip(m1,m1[1:]):
    ind1=m.hist.model_number==i
    ind2=m.hist.model_number==j
    print(m.hist.star_age[ind1],m.hist.star_age[ind2],m.hist.star_age[ind2]-m.hist.star_age[ind1])


############################################

ax= axs[1,1]
m=mp.MESA()
m.log_fold='../../data/kips/p00/60/LOGS'
m.loadHistory()
p=mp.plot()
print(m.hist.star_mass[-1])

p.plotKip3(m,xaxis='model_number',mod_min=xrng2[0],mod_max=xrng2[1],colorbar=True,show_mix_labels=False,
           mix_alpha=0.8,cmin=-18,cmax=18,fig=fig,ax=ax,show=False,yrng=[0.0,60.0],cbar_label=cbar_eng,
            mix_hatch=True,dbg=True,cpad=cpad)

t=ax.text(3200,5.0,'Oxygen',color='black',fontsize=30)
t.set_bbox(dict(facecolor='white', alpha=0.5))
t=ax.text(2900,50.0,'Helium',color='black',fontsize=30)
t.set_bbox(dict(facecolor='white', alpha=0.5))

t=ax.annotate('Carbon', xy=(2700,8),  xycoords='data',color='black',fontsize=30,
            xytext=(2780,15), textcoords='data',
            arrowprops=dict(facecolor='white', shrink=0.05),
            horizontalalignment='right', verticalalignment='top',
            )

t.set_bbox(dict(facecolor='white', alpha=0.5))

ax.set_xlabel(xlabel,labelpad=1)
ax.set_ylabel(yaxis)


ax2=ax.twiny()
ax2.set_xlim(*ax.get_xlim())
ax2.xaxis.set_ticks_position('none')
ax2.xaxis.set_label_position('top')
ax2.set_xticks(m2)
ax2.set_xticklabels(l2)

for i,j in zip(m2,m2[1:]):
    ind1=m.hist.model_number==i
    ind2=m.hist.model_number==j
    print(m.hist.star_age[ind1],m.hist.star_age[ind2],m.hist.star_age[ind2]-m.hist.star_age[ind1])

##############################################
ax = axs[1,2]
m=mp.MESA()
m.log_fold='../../data/kips/p30/60/LOGS'
m.loadHistory()
p=mp.plot()

p.plotKip3(m,xaxis='model_number',mod_min=xrng3[0],mod_max=xrng3[1],colorbar=True,show_mix_labels=False,
           mix_alpha=0.8,cmin=-18,cmax=18,fig=fig,ax=ax,show=False,yrng=[0.0,60.0],cbar_label=cbar_eng,
            mix_hatch=True,dbg=True,cpad=cpad)

t = ax.text(2900,5.0,'Oxygen',color='black',fontsize=30)
t.set_bbox(dict(facecolor='white', alpha=0.5))

t = ax.text(2900,50.0,'Helium',color='black',fontsize=30)
t.set_bbox(dict(facecolor='white', alpha=0.5))

ax.set_xlabel(xlabel,labelpad=1)
ax.set_ylabel(yaxis)


ax2=ax.twiny()
ax2.set_xlim(*ax.get_xlim())
ax2.xaxis.set_ticks_position('none')
ax2.xaxis.set_label_position('top')
ax2.set_xticks(m3)
ax2.set_xticklabels(l3)



for i,j in zip(m3,m3[1:]):
    ind1=m.hist.model_number==i
    ind2=m.hist.model_number==j
    print(m.hist.star_age[ind1],m.hist.star_age[ind2],m.hist.star_age[ind2]-m.hist.star_age[ind1])

##################################################33


def callback(m):
    return m.prof.gamma1-4/3


def hcallback(m):
    x = np.zeros(np.size(m.prof.zone))
    #ind = m.prof.du_dt_div_g < 0.0
    ind = m.prof.velocity < 0
    #ind = m.prof.v_div_cs >= 1.0
    x[:] = np.nan
    x[ind] = 1
    return x


cpad=15.0
yaxis=r"$m\, [M_{\odot}]$"

cbar_label=r'$\Gamma_1 -4/3$'

cmap='seismic'

hatch_color='black'

cmin=-0.06
cmax=0.06

############################################






ax = axs[2,0]
m=mp.MESA()
m.log_fold='../../data/kips/m30/60/LOGS'
#m.loadHistory()
m.loadProfile(num=1)
p=mp.plot()


p.plotKip3(m,xaxis='model_number',plot_type='profile',zaxis=callback,mod_min=xrng1[0],mod_max=xrng1[1],colorbar=True,show_mix_labels=False,
           fig=fig,ax=ax,show=False,yrng=[0.0,60.0],cbar_label=cbar_label,cmap=cmap,cmap_merge=False,cmin=cmin,cmax=cmax,cpad=cpad,
            hatch_color=hatch_color,hatch_func=None,cbar_extend='neither')

ax.set_xlabel(xlabel,labelpad=1)
ax.set_ylabel(yaxis)

ax2=ax.twiny()
ax2.set_xlim(*ax.get_xlim())
ax2.xaxis.set_ticks_position('none')
ax2.xaxis.set_label_position('top')
ax2.set_xticks(m1)
ax2.set_xticklabels(l1)





#######################################3


ax = axs[2,1]

m=mp.MESA()
m.log_fold='../../data/kips/p00/60/LOGS'
#m.loadHistory()
m.loadProfile(num=1)
p=mp.plot()

p.plotKip3(m,xaxis='model_number',plot_type='profile',zaxis=callback,mod_min=xrng2[0],mod_max=xrng2[1],colorbar=True,show_mix_labels=False,
           fig=fig,ax=ax,show=False,yrng=[0.0,60.0],cbar_label=cbar_label,cmap=cmap,cmap_merge=False,cmin=cmin,cmax=cmax,cpad=cpad,
            hatch_color=hatch_color,hatch_func=None,cbar_extend='neither')

ax.set_xlabel(xlabel,labelpad=1)
ax.set_ylabel(yaxis)

ax2=ax.twiny()
ax2.set_xlim(*ax.get_xlim())
ax2.xaxis.set_ticks_position('none')
ax2.xaxis.set_label_position('top')
ax2.set_xticks(m2)
ax2.set_xticklabels(l2)


##########################################3

ax = axs[2,2]

m=mp.MESA()
m.log_fold='../../data/kips/p30/60/LOGS'
#m.loadHistory()
m.loadProfile(num=1)
p=mp.plot()

p.plotKip3(m,xaxis='model_number',plot_type='profile',zaxis=callback,mod_min=xrng3[0],mod_max=xrng3[1],colorbar=True,show_mix_labels=False,
           fig=fig,ax=ax,show=False,yrng=[0.0,60.0],cbar_label=cbar_label,cmap=cmap,cmap_merge=False,cmin=cmin,cmax=cmax,cpad=cpad,
            hatch_color=hatch_color,hatch_func=None,cbar_extend='neither')

ax.set_xlabel(xlabel,labelpad=1)
ax.set_ylabel(yaxis)


ax2=ax.twiny()
ax2.set_xlim(*ax.get_xlim())
ax2.xaxis.set_ticks_position('none')
ax2.xaxis.set_label_position('top')
ax2.set_xticks(m3)
ax2.set_xticklabels(l3)


############################################################

def showGamma4(ax):
    ax.plot(p._gamma4["logRho"],p._gamma4["logT"],color='black',linewidth=3)
    ax.annotate(r'$\Gamma_{1} <4/3$', xy=(3.0,9.0), 
                        xytext=(3.5,9.3),color='black',
                        fontsize=28)

def showPgas(ax):
    lr1,lr2=ax.get_xlim()
    lt1=np.log10(3.2*10**7)+(lr1-np.log10(0.7))/3.0
    lt2=np.log10(3.2*10**7)+(lr2-np.log10(0.7))/3.0
    
    l1=ax.plot([lr1,lr2],[lt1,lt2],'--',color='black',alpha=0.5,linewidth=3,label=r'$P_{rad}\approx P_{gas}$')
    ax.legend(loc=4,fontsize=26)


def plotter(m,models,ax,labels,pad):
    cmap = plt.cm.viridis
    colors = [cmap(x/len(models)) for x in range(len(models))]

    for i, color in enumerate(colors):
        m.loadProfile(num=models[i])
        ax.plot(m.prof.logRho,m.prof.logT,linewidth=5,c=color)
        
        ax.text(m.prof.logRho[-1]+pad[i][0],m.prof.logT[-1]+pad[i][1],labels[i],color=color)

    p.setMESAPath(os.environ['MESA_DIR'])
    p._loadBurnData()
    showGamma4(ax)

    ax.tick_params(axis='x', pad=10)
    ax.tick_params(axis='y', pad=10)    
    cmap2 = mpl.colors.ListedColormap(['white'])
    cb=fig.colorbar(plt.cm.ScalarMappable(norm=plt.Normalize(vmin=0, vmax=len(models)), cmap=cmap2), ax=ax,ticks=[])
    cb.outline.set_edgecolor('white')
    cb.set_label('Time',color='white')

    ax.set_xlim(3.0,7.0)
    ax.set_ylim(8.8,9.6)
    
    showPgas(ax)

    ax.set_xlabel(r'$\log\left(\rho/\rm{\left(g\,cm^{-3}\right)}\right)$')
    ax.set_ylabel(r'$\log\,\rm{\left(T/K\right)}$')


    
m=mp.MESA()
m.log_fold='../../data/kips/m30/60/LOGS'

pad=[[0.1,0],[0.1,0],[0.0,0.02],[0.1,0]]
plotter(m,m1,axs[3,0],l1,pad)


m=mp.MESA()
m.log_fold='../../data/kips/p00/60/LOGS'

pad=[[0.1,0],[0.1,0],[0.1,0],[0.1,0]]
plotter(m,m2,axs[3,1],l2,pad)


m=mp.MESA()
m.log_fold='../../data/kips/p30/60/LOGS'

pad=[[0.1,0],[0.1,0],[0.1,0],[0.1,0]]
plotter(m,m3,axs[3,2],l3,pad)


##########################################################

fig.tight_layout()

fig.savefig('fig3.pdf',dpi=300)

