# pisn_rates.py needs to be in PYTHONPATH

import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import mesaPlot as mp
import os
import sys
import warnings
import glob
import itertools
import h5py
import scipy.stats

#pip3 install matplotlib-label-lines --user
from labellines import labelLine, labelLines

warnings.filterwarnings("ignore")

sys.path.append('../')

import pisn_rates as pisn

p=mp.plot()


plt.style.use('mesa.mplstyle')
output='./'

#masses=np.array(np.arange(30,200,2))

bases_c12_rate = ['m30','m25','m20','m15','m10','m05','p00','p05','p10','p15','p20','p25','p30']
labelsc12_rate = ['-3','-2.5','-2.0','-1.5','-1.0','-0.5','0.0','0.5','1.0','1.5','2.0','2.5','3.0']

bases_others = ['cf88','deboer','deboer_m','deboer_p','jina','kunz','nacre']
labels_others = bases_others

bases_high = ['3alow1s_h','3ahigh1s_h','o16high_h','o16low_h','tum_h_h','tum_l_h','tum_m_h']
labels_high = bases_high

bases_low = ['3alow1s_l','3ahigh1s_l','o16high_l','o16low_l','tum_h_l','tum_l_l','tum_m_l']
labels_low = bases_low

bases_mid = ['3alow1s_m','3ahigh1s_m','o16high_m','o16low_m','tum_h_m','tum_l_m','tum_m_m']
labels_mid = bases_mid

bases_split = ['corehe','shellhe','exphe']
labels_split = bases_split


masses = np.arange(30,201,1)
masses = np.array([round(i,1) for i in masses])

G=6.67428*10**-8
msun=1.9892*10**33
rsun=6.9598*10**10

data = np.genfromtxt('../../data/all_models.txt',names=True,dtype=None,encoding=None) 

#########################################33
def loadLigo():
    with h5py.File('../../data/ligo/O2/corner_AB.h5') as inp:
        O2_45 = np.array(inp['model_b']['m_max'])
    
    return O2_45



O2_45= loadLigo()

sigmas=[-3,-2.5,-2,-1.5,-1,-0.5,0.0,0.5,1.0,1.5,2.0,2.5,3.0]

mbh = []
ind=np.logical_or(data['symbol']==pisn.CC_IND,data['symbol']==pisn.PPISN_IND)&(data['bhmass'] < 110.0)
for i in bases_c12_rate:
    ind2=(data['base']==i)&ind
    mbh.append(np.nanmax(data['bhmass'][ind2]))
    
print(mbh)

bh2sig=np.poly1d(np.polyfit(mbh,sigmas,deg=4))

plt.scatter(mbh,sigmas)
x=np.linspace(35,100.0,1000)
plt.plot(x,bh2sig(x))


# Map sigma to rate
def sig2rate(filename):
    def loadfile(filename):
        return np.genfromtxt(filename,names=["t8",'rate'],dtype=None,skip_header=3)
    base=loadfile('../../input/rates/c12alpha/p00/rate_tables/rate.txt')
    x=loadfile(os.path.join('../../input/rates/c12alpha',filename,'rate_tables','rate.txt'))
    xx=x['t8']/10.0
    yy=x['rate']/base['rate']
    ind = np.nanargmin(np.abs(xx-0.3))
    return yy[ind]
    

allrates=[]
for i in bases_c12_rate:
    j=sig2rate(i)
    allrates.append(j)
    
fitsig2rate=np.poly1d(np.polyfit([float(i) for i in labelsc12_rate],allrates,deg=2))

def fit2ligo(bhs,fitsig2rate,ligo,sfact):
    data = bhs(ligo)
    res = {}
    
    def mean_confidence_interval(data, confidence=0.95):
        a = 1.0 * np.array(data)
        n = len(a)
        m, se = np.mean(a), scipy.stats.sem(a)
        h = se * scipy.stats.t.ppf((1 + confidence) / 2., n-1)
        return m
    
    
    for i in [50,90,68,95,99]:
        x = scipy.stats.norm.interval(i/100.0,loc=data.mean(),scale=np.std(data))
        res[i] = {}
        res[i]['med'] = fitsig2rate(mean_confidence_interval(data,confidence=0.95))*sfact
        res[i]['low'] = fitsig2rate(x[0])*sfact
        res[i]['high'] =fitsig2rate(x[1])*sfact
        #res[i].extend([fitsig2rate(j)*sfact for j in x])
    return res

O2_45_sfact = fit2ligo(bh2sig,fitsig2rate,O2_45,165)

print(O2_45_sfact[68])

# Library
snacre = [200.0,80.0,80.0] # NACRE
sdeboer = [140,21,21] # deboer
snacre2 = [148,27,27] # nacre2 xu 2013
sstarlib = [165.0,30,30] # starlib
shammer = [162,39,39] # Hammer 2002
sshen = [162,21,21] # Shen 2020

# Astro measurement
smwc = [290.0,15.0,15.0] # Metcalfe, T. S., Winget, D. E., & Charbonneau, P 2001 WD
smsd = [370.0,40.0,40.0] # METCALFE, SALARIS, & WINGET 2002  WD
smmet = [192.5,15,15] # Metcalfe 2003
swwt = [170,0.0,0.0] # Woosley Weaver Timmes 1993  GCE

s68 = [O2_45_sfact[68]['med'],O2_45_sfact[68]['low'],O2_45_sfact[68]['high']]
s95 = [O2_45_sfact[95]['med'],O2_45_sfact[95]['low'],O2_45_sfact[95]['high']]
s99 = [O2_45_sfact[99]['med'],O2_45_sfact[99]['low'],O2_45_sfact[99]['high']]
                       
def ploterror(r,ynum,ax,limit=False):
    rate = r[0]
    xerr = np.atleast_2d([r[1],r[2]])
    #x = xstart + (num-1)*(xend-xstart)/num

    col='black'
    
    if limit:
        ax.errorbar(r[1],ynum,xerr=20,linewidth=3,color='black',capsize=5,capthick=2,xlolims=True)
        #plotline1, caplines1, barlinecols1 = ax.errorbar(rate,ynum,xerr=np.atleast_2d([r[1],0]).T,linewidth=3,color=col,capsize=10,capthick=3)
        #caplines1[1].set_marker('|')
        #caplines1[1].set_markersize(0)
    else:
        if r[1] != 0:
            ax.scatter(rate,ynum,marker='o',s=90,color=col)
            ax.errorbar(rate,ynum,xerr=xerr.T,linewidth=3,color=col,capsize=10,capthick=3)
        else:
            ax.scatter(rate,ynum,marker='x',s=90,color=col)

        
    #ax.set_ylim(ax.get_ylim()[::-1])
    
    
fig=plt.figure(figsize=(12,9))
ax=fig.add_subplot(111)

ax.set_xlabel(r'$S_{300}\,[\rm{keV\,barns}]$',fontsize=30)

start=0

startnuc = start

ploterror(snacre,startnuc,ax)
ploterror(snacre2,startnuc+1,ax)
ploterror(shammer,startnuc+2,ax)
ploterror(sstarlib,startnuc+3,ax)
ploterror(sdeboer,startnuc+4,ax)
ploterror(sshen,startnuc+5,ax)

endnuc=startnuc+5

startast = endnuc+1

ploterror(swwt,startast,ax)
ploterror(smmet,startast+1,ax)

endast = startast+1

startme=endast+1


# print(s68,s95,s99)
ploterror(s68,startme,ax, True)
ploterror(s95,startme+1,ax, True)
ploterror(s99,startme+2,ax, True)

endme = startme+2
starttra = endme +1

ax.errorbar(79,starttra,xerr=20,linewidth=3,color='black',capsize=5,capthick=2,xlolims=True)
#ax.errorbar(s90,startme+1,xerr=20,linewidth=3,color='black',capsize=5,capthick=2,xuplims=True)
#ploterror(s50,startme,ax)

endtra = starttra



end = endtra
#ploterror(smwc,startast,ax)
#ploterror(smsd,startast+1,ax)

    
    
ax.set_xlim(-1,ax.get_xlim()[-1])

ax.fill_between(ax.get_xlim(),startme-0.5,endme+0.5,facecolor='green',alpha=0.3)


ax.fill_between(ax.get_xlim(),startnuc-0.5,endnuc+0.5,facecolor='blue',alpha=0.3)


ax.fill_between(ax.get_xlim(),startast-0.5,endast+0.5,facecolor='red',alpha=0.3)

ax.fill_between(ax.get_xlim(),starttra-0.5,endtra+0.5,facecolor='yellow',alpha=0.3)


ax.text(210,1.75,'Nuclear',fontsize=30)
ax.text(210,6.5,'Astrophysics',fontsize=30)
ax.text(210,9.0,'O1/O2',fontsize=30)
ax.text(210,11.1,'EM Transients',fontsize=30)


ylabels = [ 'NACRE','NACRE2','Hammer 05','STARLIB','Deboer 17','Shen 20',
           'Woosley 93','Metcalfe 03',
        '68\% c.i','95\% c.i','99\% c.i','PPISN']

ax.set_ylim(end+0.5,start-0.5)

ax.set_yticks(range(0,end+1))
ax.set_yticklabels(ylabels,fontsize=30)
ax.yaxis.set_ticks_position('none')

fig.savefig('./fig7.pdf',dpi=300)

