# pisn_rates.py needs to be in PYTHONPATH

import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import mesaPlot as mp
import os
import sys
import warnings
import glob
import itertools
import h5py
import scipy.stats

#pip3 install matplotlib-label-lines --user
from labellines import labelLine, labelLines

warnings.filterwarnings("ignore")

sys.path.append('../')

import pisn_rates as pisn

p=mp.plot()


plt.style.use('mesa.mplstyle')
output='./'

#masses=np.array(np.arange(30,200,2))

bases_c12_rate = ['m30','m25','m20','m15','m10','m05','p00','p05','p10','p15','p20','p25','p30']
labelsc12_rate = ['-3','-2.5','-2.0','-1.5','-1.0','-0.5','0.0','0.5','1.0','1.5','2.0','2.5','3.0']

bases_others = ['cf88','deboer','deboer_m','deboer_p','jina','kunz','nacre']
labels_others = bases_others

bases_high = ['3alow1s_h','3ahigh1s_h','o16high_h','o16low_h','tum_h_h','tum_l_h','tum_m_h']
labels_high = bases_high

bases_low = ['3alow1s_l','3ahigh1s_l','o16high_l','o16low_l','tum_h_l','tum_l_l','tum_m_l']
labels_low = bases_low

bases_mid = ['3alow1s_m','3ahigh1s_m','o16high_m','o16low_m','tum_h_m','tum_l_m','tum_m_m']
labels_mid = bases_mid

bases_split = ['corehe','shellhe','exphe']
labels_split = bases_split


masses = np.arange(30,201,1)
masses = np.array([round(i,1) for i in masses])

G=6.67428*10**-8
msun=1.9892*10**33
rsun=6.9598*10**10

data = np.genfromtxt('../../data/all_models.txt',names=True,dtype=None,encoding=None) 




def printBH(base,error=None,r=1):
    im = data['intmass']
    im[im=='']=np.nan
    intmass = im.astype(np.float)
    
    
    def inbase(b1):
        indpisn=(data['symbol']==pisn.PISN_IND)&(data['base']==b1)
        edge = np.nanmax(intmass[indpisn])

        indlow= (intmass<edge)&(data['symbol']<pisn.UPPER_IND)&(data['base']==b1)
        indhigh=(intmass>edge)&(data['symbol']<pisn.UPPER_IND)&(data['base']==b1)
        return indlow,indhigh
    
    #indlow=(data['bhmass']<cut[0])&(data['symbol']<pisn.UPPER_IND)&(data['base']==base)
    #indhigh=(data['bhmass']>cut[0])&(data['symbol']<pisn.UPPER_IND)&(data['base']==base)
    
    indlow,indhigh = inbase(base)
    
    ibhl=np.nanargmax(data['bhmass'][indlow])
    ibhh=np.nanargmin(data['bhmass'][indhigh])    
    
    bhl=data['bhmass'][indlow][ibhl]
    bhh=data['bhmass'][indhigh][ibhh]
    
    hebhl=data['intmass'][indlow][ibhl]
    hebhh=data['intmass'][indhigh][ibhh]
    
    
    #indlow_ep=(data['bhmass']<cut[1])&(data['symbol']<pisn.UPPER_IND)&(data['base']==error[0])
    #indlow_em=(data['bhmass']<cut[1])&(data['symbol']<pisn.UPPER_IND)&(data['base']==error[1])
    
    indlow_ep, indhigh_ep = inbase(error[0])
    indlow_em, indhigh_em = inbase(error[1])

    #indhigh_ep=(data['bhmass']>cut[2])&(data['symbol']<pisn.UPPER_IND)&(data['base']==error[0])
   # indhigh_em=(data['bhmass']>cut[2])&(data['symbol']<pisn.UPPER_IND)&(data['base']==error[1])
    
    #indlow_ep, indhigh = inbase(base)
    
    
    ibhl_ep = np.nanargmax(data['bhmass'][indlow_ep])
    ibhl_em = np.nanargmax(data['bhmass'][indlow_em])
    ibhh_ep = np.nanargmin(data['bhmass'][indhigh_ep])
    ibhh_em = np.nanargmin(data['bhmass'][indhigh_em])

    
    bhl_ep = data['bhmass'][indlow_ep][ibhl_ep]
    bhl_em = data['bhmass'][indlow_em][ibhl_em]
    bhh_ep = data['bhmass'][indhigh_ep][ibhh_ep]
    bhh_em = data['bhmass'][indhigh_em][ibhh_em]
    
    hebhl_ep = data['intmass'][indlow_ep][ibhl_ep]
    hebhl_em = data['intmass'][indlow_em][ibhl_em]
    hebhh_ep = data['intmass'][indhigh_ep][ibhh_ep]
    hebhh_em = data['intmass'][indhigh_em][ibhh_em]    

    
    #print('*',hebhl,hebhl_ep,hebhl_em,"**",hebhh,hebhh_ep,hebhh_em)
    #print('*',bhl,bhl_ep,bhl_em,"**",bhh,bhh_ep,bhh_em)

    #bhl_em, bhl_ep = min(bhl_em, bhl_ep),max(bhl_em, bhl_ep)
    #bhh_em, bhh_ep = min(bhh_em, bhh_ep),max(bhh_em, bhh_ep)

    def pp(mid,high,low):
        #return '$'+str(round(mid,r))+'^{+'+str(round(high-mid,r))+'}_{-'+str(round(mid-low,r))+'}$'
        ans1 = 100*(low-mid)/mid
        ans2 = 100*(high-mid)/mid
        s1=''
        s2 =''
        if ans1>0:
            s1+='\phs '
        if ans2>0:
            s2+='\phs '
        if ans1 < 10:
            s1+='\phn '
        if ans2 < 10:
            s2+='\phn '
        
        return s1+'$'+str(round(ans1,r))+'\%$',s2+'$'+str(round(ans2,r))+'\%$'
        
        
    return pp(bhl,bhl_ep,bhl_em),pp(bhh,bhh_ep,bhh_em)

    
names = ['$3\\alpha^{(1)}$ & $\\pm1\\sigma$ ','$\\carbon[12]+\\carbon[12]^{(2)}$ & $\\pm1\\sigma$ ',
         '$\\oxygen[16]+\\oxygen[16]^{(3)}$ & factor 10 ']

bbsl = ['m30','tum_m_l','m30']
bbsm = ['p00','tum_m_m','p00']
bbsh = ['p30','tum_m_h','p30']

el = [['3ahigh1s_l','3alow1s_l'],['tum_h_l','tum_l_l'],['o16high_l','o16low_l']]
em= [['3ahigh1s_m','3alow1s_m'],['tum_h_m','tum_l_m'],['o16high_m','o16low_m']]
eh= [['3ahigh1s_h','3alow1s_h'],['tum_h_h','tum_l_h'],['o16high_h','o16low_h']]
    
for n,b1,b2,b3,e1,e2,e3 in zip(names,bbsl,bbsm,bbsh,el,em,eh):
    a1,a2=printBH(b1,e1)
    a3,a4=printBH(b2,e2)
    a5,a6=printBH(b3,e3)

    print(' & '.join([n,a1[0],a2[0],a3[0],a4[0],a5[0],a6[0]]),end=' \\\\')
    print()
    print(' & '.join([' & ',a1[1],a2[1],a3[1],a4[1],a5[1],a6[1]]),end=' \\\\')
    print()
    print('\\hline')
